package be.cylab.java.wowa.training;

import info.debatty.java.aggregation.WOWA;

import java.util.Arrays;
import java.util.List;

/**
 * Class of Solution.
 * Solution with distance performance evaluation criterion
 */
public class SolutionDistance implements Comparable<SolutionDistance> {

    private double[] weights_w;
    private double[] weights_p;
    private double distance = Double.POSITIVE_INFINITY;

    /**
     * SolutionDistance constructor.
     * Needs weight number as parameter
     * @param weights_number
     */
    public SolutionDistance(final int weights_number) {
        this.weights_w = new double[weights_number];
        this.weights_p = new double[weights_number];
    }

    /**
     * @return
     */
    @Override
    final public String toString() {
        return "SolutionDistance{"
                + "weights_w=" + Arrays.toString(weights_w)
                + ", weights_p=" + Arrays.toString(weights_p)
                + ", distance=" + distance
                + '}';
    }

    /**
     * @param data
     * @param expected
     */
    void computeScoreTo(final List<double[]> data, final double[] expected) {

        this.distance = 0;
        for (int i = 0; i < data.size(); i++) {
            double[] vector = data.get(i);
            double target_value = expected[i];
            WOWA wowa = new WOWA(this.weights_w, this.weights_p);
            double aggregated_value = wowa.aggregate(vector);
            this.distance += Math.pow(target_value - aggregated_value, 2);
        }
        this.distance = Math.sqrt(this.distance);

    }

    /**
     * @param probability
     */
    void randomlyMutateWithProbability(final double probability) {
        double tos = Math.random();
        if (tos > probability) {
            // do nothing
            return;
        }

        //new weight value and position
        double new_weight_value = Math.random();
        int random_index = Utils.randomInteger(0, this.weights_p.length - 1);
        //Select w or p weights
        int weight_selection = Utils.randomInteger(0, 1);
        if (weight_selection == 0) {
            this.weights_w[random_index] = new_weight_value;
        } else {
            this.weights_p[random_index] = new_weight_value;
        }
        this.normalize();

    }

    /**
     * @param solution
     * @return
     */
    @Override
    public int compareTo(final SolutionDistance solution) {
        return this.getDistance() > solution.getDistance() ? -1
                : this.getDistance() < solution.getDistance() ? 1 : 0;
    }

    /**
     *
     */
    void normalize() {
        this.weights_w = Utils.normalizeWeights(this.weights_w);
        this.weights_p = Utils.normalizeWeights(this.weights_p);
    }

    /**
     * @return
     */
    double getDistance() {
        return distance;
    }

    double[] getWeightsW() {
        return this.weights_w;
    }

    double[] getWeightsP() {
        return this.weights_p;
    }
}
